import { ModeEnum, Setting, MarkStore, Task } from "@/types";
import { groupBy, sortBy } from "lodash";
import { reactive, watch } from "vue";

const obj = {
  setting: {
    mode: "TRACK",
    examType: "SCAN_IMAGE",
    forceMode: false,
    sheetView: false,
    sheetConfig: [],
    enableAllZero: false,
    fileServer: "",
    marker: <Setting["marker"]>{},
    subject: <Setting["subject"]>{},
    forceSpecialTag: false,
    uiSetting: {
      "answer.paper.scale": 1,
      "score.board.collapse": false,
      "normal.mode": "keyboard",
    },
    statusValue: "FORMAL",
    problemTypes: [],
    groupNumber: 0,
    topCount: 0,
    splitConfig: [],
    prefetchCount: 3,
    startTime: 0,
    endTime: 0,
  },
  status: <MarkStore["status"]>{},
  groups: [],
  tasks: [],
  currentMarkResult: undefined,
  currentTask: <Task>{},
  currentQuestion: undefined,
  currentScore: undefined,
  markResults: [],
  historyOpen: false,
  MarkBoardTrackCollapse: false,
  historyTasks: [],
} as MarkStore;

/** 保存setting和task */
export function saveSetting(settingRes: Setting) {
  store.setting = settingRes;
}

/** 切换评分模式 */
export function toggleMode() {
  store.setting.mode =
    store.setting.mode === ModeEnum.TRACK ? ModeEnum.COMMON : ModeEnum.TRACK;
  // 切换模式会清除当前任务的打分
}

export function findCurrentTaskMarkResult() {
  if (!store.currentTask) return;
  const { libraryId, studentId } = store.currentTask;
  const statusValue = store.setting.statusValue;
  return store.markResults.find(
    (m) =>
      m.libraryId === libraryId &&
      m.studentId === studentId &&
      m.statusValue === statusValue
  );
}

/** 给当前任务评分 */
export function markScore() {}

export const store = reactive(obj);

watch(
  () => store.currentTask,
  () => {
    const m = findCurrentTaskMarkResult();
    store.currentMarkResult = m;
  }
);

watch(
  () => store.currentMarkResult?.trackList,
  () => {
    const markResult = findCurrentTaskMarkResult();
    if (
      markResult &&
      store.currentMarkResult &&
      store.currentMarkResult.trackList.length > 0
    ) {
      const scoreGroups = groupBy(
        markResult.trackList,
        (obj) =>
          (obj.mainNumber + "").padStart(10, "0") +
          obj.subNumber.padStart(10, "0")
      );
      const questionWithScore = Object.entries(scoreGroups);
      const questionWithTotalScore = questionWithScore.map((v) => [
        v[0],
        v[1].reduce((acc, c) => (acc += c.score), 0),
      ]);
      const questionWithTotalScoreSorted = sortBy(
        questionWithTotalScore,
        (obj) => obj[0]
      );
      const scoreList = questionWithTotalScoreSorted.map((s) => s[1]);
      // console.log(
      //   scoreGroups,
      //   questionWithScore,
      //   questionWithTotalScore,
      //   questionWithTotalScoreSorted,
      //   scoreList
      // );
      const cq = store.currentQuestion;
      if (cq) {
        cq.score =
          markResult.trackList
            .filter(
              (v) =>
                v.mainNumber === cq.mainNumber && v.subNumber === cq.subNumber
            )
            .map((v) => v.score)
            .reduce((acc, v) => (acc += v * 100), 0) / 100;
      }
      markResult.scoreList = scoreList as number[];
      // const sortScore = orderBy(markResult.trackList, ['mainNumber', 'subNumber', 'score']);
      // markResult.scoreList = sortScore.reduce((acc, pre) => {
      //   if(pre.mainNumber === cur.mainNumber && pre.subNumber === cur.subNumber) {
      //     acc[acc.length-1] += cur.score
      //   }
      // }, [0])
      markResult.markerScore =
        markResult.scoreList.reduce((acc, v) => (acc += v * 100), 0) / 100;

      // console.log(markResult.scoreList, markResult.markerScore);
    }
  },
  { deep: true }
);

// 普通模式更新分数时
watch(
  () => store.currentQuestion?.score,
  () => {
    if (store.setting.mode === ModeEnum.COMMON) {
      const markResult = findCurrentTaskMarkResult();
      if (markResult && store.currentTask) {
        const scoreList = store.currentTask.questionList.map(
          (q) => q.score || 0
        );
        markResult.scoreList = scoreList as number[];
        markResult.markerScore =
          scoreList.reduce((acc, v) => (acc += v * 100), 0) / 100;
      }
    }
  }
);
